package de.uni_marburg.powersort.benchmark;

import de.uni_marburg.powersort.TimSort;

import java.util.Arrays;
import java.util.List;
import java.util.function.Supplier;

/**
 * Custom benchmark.
 */
public class Main {
    public static void main(final String[] args) {
        final SortImpl[] sortImplementations = getSortImplementations();
        final List<Supplier<Object[]>> sortInputSuppliers = getSortInputSuppliers();

        for (Supplier<Object[]> sortInputSupplier : sortInputSuppliers) {
            for (SortImpl sortImplementation : sortImplementations) {
                Object[] sortInput = sortInputSupplier.get();

                final long startTime = System.nanoTime();
                sortImplementation.sort(sortInput);
                final long stopTime = System.nanoTime();
                final long duration = stopTime - startTime;

                System.out.println(duration + "," + sortImplementation.title + "," + sortImplementation.description);
            }
        }
    }

    static SortImpl[] getSortImplementations() {
        return new SortImpl[]{
                new SortImpl("DummySort", "DummySort.sort(a,c) uses Arrays.sort(a,c) internally.") {
                    @Override
                    public void sort(Object[] a) {
                        DummySort.sort(a, 0, a.length, NaturalOrder.INSTANCE, null, 0, 0);
                    }
                },
                new SortImpl("DummySort2", "DummySort2.sort(a,c) uses Arrays.sort(a,c) internally. It calles that method multiple times making this implementation really slow.") {
                    @Override
                    public void sort(Object[] a) {
                        DummySort2.sort(a, 0, a.length, NaturalOrder.INSTANCE, null, 0, 0);
                    }
                },
                //
                new SortImpl("TimSort", "Copy of JDK23 TimSort.java") {
                    @Override
                    public void sort(Object[] a) {
                        TimSort.sort(a, 0, a.length, NaturalOrder.INSTANCE, null, 0, 0);
                    }
                }
        };
    }

    static List<Supplier<Object[]>> getSortInputSuppliers() {
        return Arrays.asList(
                () -> new DummyComparable2[]{new DummyComparable2(0), new DummyComparable2(1)},
                () -> new DummyComparable2[]{new DummyComparable2(2), new DummyComparable2(3)}
        );
    }
}
